@@ -25,9 +25,9 @@
 #include <QPixmap>
 #include <QStyleOptionGraphicsItem>
 #include <QGraphicsSceneDragDropEvent>
+#include <QGraphicsLinearLayout>
 #include <QPen>
 #include <QCheckBox>
-#include <QTimer>
 #include <QFileInfo>
 #include <QStandardItemModel>
 
@@ -42,6 +42,8 @@
 
 #include <Plasma/PaintUtils>
 #include <Plasma/DataEngine>
+#include <Plasma/ToolButton>
+#include <Plasma/Frame>
 
 #include <math.h>
 
@@ -52,19 +54,26 @@
 Frame::Frame(QObject *parent, const QVariantList &args)
         : Plasma::Applet(parent, args),
         m_configDialog(0),
-        m_openPicture(0)
+        m_slideFrame(0)
 {
     setHasConfigurationInterface(true);
     setAcceptDrops(true);
     setAcceptsHoverEvents(true);
     setCacheMode(QGraphicsItem::DeviceCoordinateCache);
     resize(400, 300);
+    //make size()==contentssize(), resolves auto-shrinking once for all
+    setContentsMargins(0, 0, 0, 0);
     m_mySlideShow = new SlideShow(this);
     if (args.count()) {
         m_currentUrl = args.value(0).toString();
     } else {
-        m_currentUrl = KUrl("Default");
+        m_currentUrl = KUrl();
     }
+    setAssociatedApplicationUrls(m_currentUrl);
+
+    m_updateTimer = new QTimer(this);
+    m_updateTimer->setSingleShot(true);
+    connect(m_updateTimer, SIGNAL(timeout()), this, SLOT(delayedUpdateSize()));
 }
 
 Frame::~Frame()
@@ -74,7 +83,7 @@ Frame::~Frame()
 void Frame::init()
 {
     bool frameReceivedUrlArgs = false;
-    if (m_currentUrl != KUrl("Default")) {
+    if (!m_currentUrl.isEmpty()) {
         frameReceivedUrlArgs = true;
     }
 
@@ -92,7 +101,8 @@ void Frame::init()
     m_slideShowPaths = cg.readEntry("slideshow paths", QStringList());
     m_slideshowTime = cg.readEntry("slideshow time", 60); // default to 1 minute
     m_currentUrl = cg.readEntry("url", m_currentUrl);
-    m_potdProvider = cg.readEntry("potdProvider", "");
+    setAssociatedApplicationUrls(m_currentUrl);
+    m_potdProvider = cg.readEntry("potdProvider", "wcpotd");
     m_potd = cg.readEntry("potd", false);
 
     // Frame & Shadow dimensions
@@ -109,29 +119,10 @@ void Frame::init()
     }
 
     m_menuPresent = false;
-}
 
-void Frame::updateMenu()
-{
-    if (hasAuthorization("LaunchApp") && ! (m_menuPresent || m_potd || (m_currentUrl.path() == "Default" && m_mySlideShow->currentUrl() == "Default"))) {
-        kDebug() << "Current url: " << m_currentUrl.url();
-        m_openPicture = new QAction(SmallIcon("image-x-generic"), i18n("&Open Picture..."), this);       
-        m_actions.append(m_openPicture);
-        connect(m_openPicture, SIGNAL(triggered(bool)), this , SLOT(slotOpenPicture()));
-        m_menuPresent = true;
-    } else {
-        if (m_menuPresent && m_potd) {
-            m_actions.removeAll(m_openPicture);
-            delete m_openPicture;
-            m_openPicture = 0;
-            m_menuPresent = false;
-        }
-    }
-}
-
-QList<QAction*> Frame::contextualActions()
-{
-    return m_actions;
+    QAction *openAction = action("run associated application");
+    openAction->setIcon(SmallIcon("image-x-generic"));
+    openAction->setText(i18n("&Open Picture..."));
 }
 
 void Frame::slotOpenPicture()
@@ -147,7 +138,7 @@ void Frame::slotOpenPicture()
         url = m_currentUrl;
     }
 
-    if (url.path() != "Default") {
+    if (!url.path().isEmpty()) {
         new KRun(url, 0);
     }
 }
@@ -169,60 +160,242 @@ void Frame::constraintsEvent(Plasma::Constraints constraints)
             resize(contentSizeHint());
             emit appletTransformedItself();
         }
-
-        updatePicture();
+        m_updateTimer->start(400);
     }
 
     if (constraints & Plasma::SizeConstraint) {
         //If on panel, keep geometry to 4:3 ratio
         if(formFactor() == Plasma::Vertical) {
-            setMinimumSize(QSizeF(0, boundingRect().width()/1.33));
-            setMaximumSize(QSizeF(-1, boundingRect().width()/1.33));
+            setMinimumSize(QSizeF(0, contentsRect().width()/1.33));
+            setMaximumSize(QSizeF(-1, contentsRect().width()/1.33));
         } else if(formFactor() == Plasma::Horizontal) {
-            setMinimumSize(QSizeF(boundingRect().height()*1.33,0));
-            setMaximumSize(QSizeF(boundingRect().height()*1.33,-1));
+            setMinimumSize(QSizeF(contentsRect().height()*1.33,0));
+            setMaximumSize(QSizeF(contentsRect().height()*1.33,-1));
         } else {
-            setMinimumSize(QSizeF());
+            int min = 48;
+            if (m_shadow) {
+                min += m_swOutline;
+            }
+            if (m_frame) {
+                min += m_frameOutline;
+            }
+            setMinimumSize(QSizeF(min, min));
             setMaximumSize(QSizeF());
         }
+
+        if (m_slideShow) {
+            checkSlideFrame();
+
+            int x = contentsRect().center().x() - (m_slideFrame->size().width() / 2);
+            int y = contentsRect().bottom() - m_slideFrame->size().height() - 5;
+            m_slideFrame->setPos(x, y);
+        }
+
+        updatePicture();
+        m_updateTimer->start(400);
     }
 }
 
 QSizeF Frame::contentSizeHint() const
 {
     if (!m_pictureSize.isEmpty() && (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter)){
-        const qreal maxSize = geometry().width() > geometry().height() ? geometry().width() : geometry().height();
+        const qreal maxSize = qMax(contentsRect().width(), contentsRect().height());
         QSize size = m_pictureSize;
         size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
         return size;
     } else {
-        return geometry().size();
+        return contentsRect().size();
+    }
+}
+
+QSizeF Frame::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
+{
+    if (which != Qt::PreferredSize) {
+        return Applet::sizeHint(which, constraint);
+    } else {
+        return m_pictureSize;
     }
 }
 
 void Frame::updatePicture()
 {
-    QImage picture = m_mySlideShow->image();
-    m_pictureSize = picture.size();
+    m_pictureSize = m_mySlideShow->image().size();
     QSizeF sizeHint = contentSizeHint();
-    if (geometry().size() != sizeHint) {
-        resize(sizeHint);
-        emit appletTransformedItself();
+    int frameLines = qMin(m_frameOutline, (int)(sizeHint.height()/10));
+    const QSize contentsSize = sizeHint.toSize();
+
+    if (m_currentUrl.url().isEmpty() && m_mySlideShow->currentUrl().isEmpty()) {
+        setAssociatedApplicationUrls(KUrl::List());
+    } else {
+        setAssociatedApplicationUrls(m_mySlideShow->currentUrl());
     }
-    updateMenu();
+
+    emit sizeHintChanged(Qt::PreferredSize);
+    resize(contentSizeHint());
+
+    kDebug() << "Rendering picture";
+
+    // create a QPixmap which can be drawn in paintInterface()
+    QPixmap picture = m_mySlideShow->image();
+
+    if (picture.isNull()) {
+        return;
+    }
+
+    m_pixmap = QPixmap(contentsSize);
+    m_pixmap.fill(Qt::transparent);
+    QPainter *p = new QPainter();
+    p->begin(&m_pixmap);
+
+    int roundingFactor = qMin(qreal(sizeHint.height() / 10), qreal(12.0)) * m_roundCorners;
+    int swRoundness = roundingFactor + frameLines / 2 * m_frame * m_roundCorners;
+
+    QRectF frameRect(QPoint(0, 0), contentsSize);
+    frameRect.adjust(m_swOutline, m_swOutline, -m_swOutline, -m_swOutline); //Pretty useless.
+
+    QSizeF scaledSize = frameRect.size();
+    scaledSize.scale(frameRect.size(), Qt::KeepAspectRatio);
+    frameRect = QRectF(QPoint(frameRect.x() + (frameRect.width() - scaledSize.width()) / 2,
+                      frameRect.y() + (frameRect.height() - scaledSize.height()) / 2), scaledSize);
+
+    QRectF shadowRect;
+    if (m_frame) {
+        shadowRect = frameRect.adjusted(-frameLines, -frameLines,
+                                        frameLines, frameLines);
+    } else {
+        shadowRect = frameRect;
+    }
+
+    // The frame path. It will be used to draw the frame and clip the image.
+    QPainterPath framePath = Plasma::PaintUtils::roundedRectangle(frameRect, roundingFactor);
+
+    p->setRenderHint(QPainter::SmoothPixmapTransform, true);
+    p->setRenderHint(QPainter::Antialiasing, true);
+
+    // Shadow
+    if (m_shadow) {
+        // The shadow is a couple of lines with decreasing opacity painted around the path
+        p->setBrush(Qt::NoBrush);
+        QPen pen = QPen(Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin);
+        int shadowLines = qMin((int)(sizeHint.height() / 6), m_swOutline);
+
+        // The shadow is drawn from inside to the outside
+        shadowRect.adjust(+shadowLines, +shadowLines, -shadowLines, -shadowLines);
+
+        // Make the path to paint the frame in a bit smaller so it's inside the shadow
+        frameRect.adjust(+shadowLines, +shadowLines, -shadowLines, -shadowLines);
+        framePath = Plasma::PaintUtils::roundedRectangle(frameRect, roundingFactor);
+
+        // Paint the shadow's lines around the picture
+        for (int i = m_swOutline - shadowLines; i <= m_swOutline; i += 1) {
+            // It's important to change the opacity using the QColor, as
+            // QPainter->setOpacity() kills performance
+            qreal opacity = 0.7 * exp(-(i / (double)(m_swOutline / 3)));
+            pen.setColor(QColor(0, 0, 0, opacity * 254));
+            p->setPen(pen);
+            QPainterPath tr = Plasma::PaintUtils::roundedRectangle(shadowRect, swRoundness + i);
+            p->drawPath(tr);
+            shadowRect.adjust(-1, -1, + 1, + 1);
+        }
+    }
+
+    p->setBrush(Qt::NoBrush);
+
+    // Frame
+    if (m_frame) {
+        m_frameColor.setAlphaF(0.5);
+        // The frame is painted twice as thick, the inner half lies behind the picture
+        // This is important to not make the corners look "cut out" when rounded
+        p->setPen(QPen(m_frameColor, frameLines * 2, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
+        p->drawPath(framePath);
+    }
+
+    // Picture
+
+    // We save the painter here so the clipping only happens on the pixmap,
+    // not on pixmap, frame, shadow, etc
+    p->save();
+    if (m_roundCorners) {
+        p->setClipPath(framePath);
+    }
+
+    // Respect the smoothScaling setting
+    p->setRenderHint(QPainter::SmoothPixmapTransform, m_smoothScaling);
+    // draw our pixmap into the computed rectangle
+    p->drawPixmap(frameRect.toRect(), m_mySlideShow->image());
+    p->restore();
+
+    p->setRenderHint(QPainter::SmoothPixmapTransform, true);
+    // black frame
+    if (m_frame) {
+        p->setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+        p->drawPath(framePath);
+    } else if (m_roundCorners) {
+        p->setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+        p->drawPath(framePath);
+    }
+
+    // Paint status text on top of it all
+    if (!m_mySlideShow->message().isEmpty()) {
+        int dist = frameRect.width() / 10;
+        QRect bgRect = frameRect.adjusted(dist-1, dist-1, -dist, -dist).toRect();
+
+        // The text's background rounded rectangle
+        QPainterPath bgPath = Plasma::PaintUtils::roundedRectangle(bgRect, bgRect.height()/15);
+
+        QColor c = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor);
+        c.setAlphaF(.3);
+        QColor outline = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
+        outline.setAlphaF(.5);
+        p->setBrush(c);
+        p->setPen(outline);
+        p->setPen(QPen(outline, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+        p->drawPath(bgPath);
+        QString message = m_mySlideShow->message();
+
+        // Set the font and draw text
+        p->setRenderHint(QPainter::Antialiasing);
+        QFont textFont = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
+        textFont.setPixelSize(qMax(KGlobalSettings::smallestReadableFont().pixelSize(), bgRect.height() / 6));
+        p->setFont(textFont);
+
+        QTextOption option;
+        option.setAlignment(Qt::AlignCenter);
+        option.setWrapMode(QTextOption::WordWrap);
+        p->setPen(QPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor), 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+        p->drawText(bgRect, message, option);
+    }
+
+    p->end();
+    delete p;
     update();
 }
 
+void Frame::nextPicture()
+{
+    m_mySlideShow->setUpdateInterval(0);
+    m_mySlideShow->nextPicture();
+    m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
+}
+
+void Frame::previousPicture()
+{
+    m_mySlideShow->setUpdateInterval(0);
+    m_mySlideShow->previousPicture();
+    m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
+}
+
 void Frame::addDir()
 {
-    KDirSelectDialog dialog(KUrl(), true);
-    if (dialog.exec()) {
-        QString path = dialog.url().url();
+    QPointer<KDirSelectDialog> dialog = new KDirSelectDialog(KUrl(), true);
+    if (dialog->exec()) {
+        QString path = dialog->url().url();
         if (!m_slideShowPaths.contains(path)) {
             m_configDialog->imageUi.slideShowDirList->addItem(path);
         }
         updateButtons();
     }
+    delete dialog;
 }
 
 void Frame::removeDir()
@@ -262,7 +435,7 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     }
 
     parent->addPage(m_configDialog->imageSettings, i18n("Image"), icon());
-    parent->addPage(m_configDialog->appearanceSettings, i18n("Appearance"), icon());
+    parent->addPage(m_configDialog->appearanceSettings, i18n("Appearance"), "preferences-desktop-theme");
     parent->setDefaultButton(KDialog::Ok);
     parent->showButtonSeparator(true);
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
@@ -330,6 +503,7 @@ void Frame::configAccepted()
     m_random = m_configDialog->random();
     cg.writeEntry("random", m_random);
     m_currentUrl = m_configDialog->currentUrl();
+    setAssociatedApplicationUrls(m_currentUrl);
     cg.writeEntry("url", m_currentUrl);
     cg.writeEntry("slideshow", m_slideShow);
     m_recursiveSlideShow = m_configDialog->imageUi.recursiveCheckBox->checkState() == Qt::Checked ? true : false;
@@ -358,9 +532,6 @@ void Frame::configAccepted()
     cg.writeEntry("potd", m_potd);
 
     initSlideShow();
-    
-    // Creates the menu if the settings have changed from "Default" to sth. else
-    updateMenu();
 
     emit configNeedsSaving();
 }
@@ -371,7 +542,6 @@ void Frame::stopPotd()
     QDate mCurrentDate = QDate::currentDate();
     const QString identifier = m_potdProvider + ':' + mCurrentDate.toString(Qt::ISODate);
     engine->disconnectSource(identifier, m_mySlideShow);
-    m_potd = false;
 }
 
 void Frame::initSlideShow()
@@ -379,8 +549,8 @@ void Frame::initSlideShow()
     m_mySlideShow->setUpdateInterval(0);
 
     if (m_slideShow) {
-        m_mySlideShow->setDirs(m_slideShowPaths, m_recursiveSlideShow);
         m_mySlideShow->setRandom(m_random);
+        m_mySlideShow->setDirs(m_slideShowPaths, m_recursiveSlideShow);
         m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
     } else if (m_potd) {
         Plasma::DataEngine *engine = dataEngine("potd");
@@ -395,9 +565,8 @@ void Frame::initSlideShow()
         m_mySlideShow->setImage(m_currentUrl.url());
     }
 
-    if (!m_potd) {
-        updatePicture();
-    }
+    updatePicture();
+
 }
 
 void Frame::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
@@ -422,18 +591,16 @@ void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
     if (droppedUrl.isLocalFile() && QFileInfo(droppedUrl.path()).isDir()) {
         m_slideShowPaths.clear();
         m_slideShowPaths.append(droppedUrl.path());
-        if (!m_slideShow) {
-            m_slideShow = true;
-        }
+        m_slideShow = true;
     } else {
         kDebug() << "Remote URL" << droppedUrl.url();
         m_currentUrl = droppedUrl;
-        if (m_slideShow) {
-            m_slideShow = false;
-        }
+        setAssociatedApplicationUrls(m_currentUrl);
+        m_slideShow = false;
     }
 
     stopPotd();
+    m_potd = false;
     initSlideShow();
 
     KConfigGroup cg = config();
@@ -443,95 +610,68 @@ void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
     emit configNeedsSaving();
 }
 
+void Frame::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
+{
+    if (m_slideShow) {
+        checkSlideFrame();
+        m_slideFrame->show();
+    }
+
+    Applet::hoverEnterEvent(event);
+}
+
+void Frame::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
+{
+    if (m_slideShow) {
+        checkSlideFrame();
+        m_slideFrame->hide();
+    }
+
+    Applet::hoverLeaveEvent( event );
+}
+
+void Frame::checkSlideFrame()
+{
+    if (m_slideFrame)
+        return;
+
+    m_slideFrame = new Plasma::Frame( this );
+    m_slideFrame->setZValue( 10 );
+
+    m_backButton = new Plasma::ToolButton(m_slideFrame);
+    m_backButton->setImage("widgets/arrows", "left-arrow");
+    m_backButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+    m_backButton->setMaximumSize(IconSize(KIconLoader::MainToolbar), IconSize(KIconLoader::MainToolbar));
+    connect(m_backButton, SIGNAL(clicked()), this , SLOT(previousPicture()));
+
+    m_nextButton = new Plasma::ToolButton(m_slideFrame);
+    m_nextButton->setImage("widgets/arrows", "right-arrow");
+    m_nextButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+    m_nextButton->setMaximumSize(IconSize(KIconLoader::MainToolbar), IconSize(KIconLoader::MainToolbar));
+    connect(m_nextButton, SIGNAL(clicked()), this , SLOT(nextPicture()));
+
+    QGraphicsLinearLayout *buttonsLayout = new QGraphicsLinearLayout();
+    buttonsLayout->addItem(m_backButton);
+    buttonsLayout->addItem(m_nextButton);
+    m_slideFrame->setLayout(buttonsLayout);
+    buttonsLayout->activate();
+
+    m_slideFrame->setFrameShadow( Plasma::Frame::Raised );
+    m_slideFrame->hide();
+
+    constraintsEvent(Plasma::SizeConstraint);
+}
+
 void Frame::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &rect)
 {
+    //kDebug() << "Paint!";
     Q_UNUSED(option)
     if (m_slideShow) {
         // temporarily suspend the slideshow to allow time for loading the image
         m_mySlideShow->setUpdateInterval(0);
     }
 
-    QImage picture = m_mySlideShow->image();
-    const QSize contentsSize = rect.size();
-
-    if (picture.isNull()) {
-        if (m_slideShow) {
-            // unsuspend the slideshow to allow time for loading the image
-            m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
-        }
-        return;
-    }
-
-    int roundingFactor = 12 * m_roundCorners;
-    int swRoundness = roundingFactor + m_frameOutline / 2 * m_frame * m_roundCorners;
-
-    QRect frameRect(QPoint(0, 0), contentsSize);
-    frameRect.adjust(m_swOutline, m_swOutline, -m_swOutline, -m_swOutline); //Pretty useless.
-
-    Qt::TransformationMode transformationMode = m_smoothScaling ? Qt::SmoothTransformation : Qt::FastTransformation;
-    //TODO check if correct
-    QImage scaledImage = picture.scaled(frameRect.size(), Qt::KeepAspectRatio, transformationMode);
-    frameRect = QRect(QPoint(frameRect.x() + (frameRect.width() - scaledImage.width()) / 2,
-                             frameRect.y() + (frameRect.height() - scaledImage.height()) / 2), scaledImage.size());
-
-    QRect shadowRect;
-    if (m_frame) {
-        shadowRect = frameRect.adjusted(-m_frameOutline + 1, -m_frameOutline + 1,
-                                        m_frameOutline - 1, m_frameOutline - 1);
-    } else {
-        shadowRect = frameRect;
-    }
-
-    // choose where to draw.
-
-    // The frame path. It will be used to draw the frame and clip the image.
-    QPainterPath framePath = Plasma::PaintUtils::roundedRectangle(frameRect, roundingFactor);
-
-    p->setRenderHint(QPainter::SmoothPixmapTransform, true);
-    p->setRenderHint(QPainter::Antialiasing, true);
-
-    // Shadow
-    // TODO faster. I'd like to use it on liveTransform.
-    if (m_shadow) {
-        p->setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin));
-        p->setBrush(Qt::NoBrush);
-        for (int i = 0; i <= m_swOutline; i += 1) {
-            p->setOpacity(0.7 * exp(-(i / (double)(m_swOutline / 3))));
-            QPainterPath tr = Plasma::PaintUtils::roundedRectangle(shadowRect, swRoundness + i);
-            p->drawPath(tr);
-            shadowRect.adjust(-1, -1, + 1, + 1);
-        }
-    }
-
-    p->setBrush(Qt::NoBrush);
-
-    // Frame
-    if (m_frame) {
-        p->setOpacity(0.5);
-        p->setPen(QPen(m_frameColor, m_frameOutline * 2, Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
-        p->drawPath(framePath);
-    }
-
-    p->setOpacity(1);
-
-    // Picture
-    p->save();
-    if (m_roundCorners) {
-        p->setClipPath(framePath);
-    }
-
-    // scale and center
-    p->drawImage(frameRect, scaledImage);
-    p->restore();
-
-    // black frame
-    if (m_frame) {
-        p->setPen(QPen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
-        p->drawPath(framePath);
-    } else if (m_roundCorners) {
-        p->setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
-        p->drawPath(framePath);
-    }
+    p->drawPixmap(rect, m_pixmap, rect);
 
     if (m_slideShow) {
         // unsuspend the slideshow to allow time for loading the image
@@ -539,4 +679,10 @@ void Frame::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
     }
 }
 
-#include "frame.moc"
+void Frame::delayedUpdateSize()
+{
+    resize(contentSizeHint());
+    emit appletTransformedItself();
+}
+
+#include "frame.moc"
\ No newline at end of file
