@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser/Library General Public License version 2,
@@ -18,9 +18,10 @@
  */
 
 #include "FlipLayout.h"
-#include <QSet>
-#include <QMap>
+
 #include <QGraphicsWidget>
+#include <QMap>
+#include <QSet>
 
 namespace Lancelot
 {
@@ -45,7 +46,7 @@ FlipLayoutManager::~FlipLayoutManager()
     delete d;
 }
 
-FlipLayoutManager * FlipLayoutManager::instance()
+FlipLayoutManager * FlipLayoutManager::self()
 {
     if (m_instance == NULL) {
         m_instance = new FlipLayoutManager();
