@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser/Library General Public License version 2,
@@ -20,13 +20,13 @@
 #ifndef LANCELOT_PANEL_H
 #define LANCELOT_PANEL_H
 
-#include <lancelot/lancelot_export.h>
+#include <QtGui/QIcon>
+
 #include <plasma/plasma.h>
 
+#include <lancelot/lancelot_export.h>
 #include <lancelot/widgets/Widget.h>
 
-#include <QtGui/QIcon>
-
 namespace Lancelot
 {
 
@@ -44,8 +44,8 @@ class LANCELOT_EXPORT Panel: public Lancelot::Widget
     Q_PROPERTY ( QString title READ title WRITE setTitle ) L_EXTRA ( i18n )
     Q_PROPERTY ( bool showingTitle READ isShowingTitle WRITE setShowingTitle )
 
-    L_WIDGET
-    L_INCLUDE(lancelot/widgets/Panel.h QIcon QSize QString)
+    // @puck L_WIDGET
+    // @puck L_INCLUDE(lancelot/widgets/Panel.h QIcon QSize QString)
 
 public:
     /**
@@ -136,14 +136,13 @@ public:
      */
     qreal borderSize(Plasma::MarginEdge edge) const;
 
-    L_Override void setGeometry(const QRectF & geometry);
-    L_Override void setGroup(WidgetGroup * group = NULL);
-    // L_Override void groupUpdated();
+    L_Override void setGroup(Group * group = NULL);
 
 protected:
     // L_Override void paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget = 0);
     L_Override QSizeF sizeHint(Qt::SizeHint which,
             const QSizeF & constraint = QSizeF()) const;
+    L_Override void resizeEvent(QGraphicsSceneResizeEvent * event);
 
 private:
     class Private;
