@@ -19,33 +19,25 @@
  ***************************************************************************/
 
 #include "microblog.h"
+#include "postwidget.h"
 
-#include <QApplication>
-#include <QGridLayout>
-#include <QPushButton>
-#include <QSpinBox>
-#include <QCheckBox>
 #include <QFontMetrics>
 #include <QGraphicsView>
 #include <QGraphicsLinearLayout>
-#include <QGraphicsGridLayout>
-#include <QGraphicsProxyWidget>
+#include <QGraphicsAnchorLayout>
 #include <QAction>
-#include <QLabel>
+#include <QTimer>
 
 #include <KColorScheme>
 #include <KConfigDialog>
-#include <KDateTime>
 #include <KDebug>
 #include <KIcon>
-#include <KLineEdit>
 #include <KMessageBox>
-#include <KRun>
-#include <KSharedConfig>
 #include <KStringHandler>
-#include <KTextEdit>
 #include <KTextBrowser>
+#include <KTextEdit>
 #include <KWallet/Wallet>
+#include <KToolInvocation>
 
 #include <Plasma/Svg>
 #include <Plasma/Theme>
@@ -54,6 +46,7 @@
 #include <Plasma/FlashingLabel>
 #include <Plasma/IconWidget>
 #include <Plasma/SvgWidget>
+#include <Plasma/TabBar>
 #include <Plasma/TextBrowser>
 #include <Plasma/ScrollWidget>
 #include <Plasma/TextEdit>
@@ -67,14 +60,13 @@ MicroBlog::MicroBlog(QObject *parent, const QVariantList &args)
     : Plasma::PopupApplet(parent, args),
       m_graphicsWidget(0),
       m_newTweets(0),
+      m_lastMode(0),
       m_service(0),
       m_profileService(0),
-      m_statusService(0),
       m_lastTweet(0),
       m_wallet(0),
       m_walletWait(None),
       m_colorScheme(0),
-      m_tz(KTimeZone::utc()),
       m_showTweetsTimer(0)
 {
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
@@ -172,7 +164,7 @@ QGraphicsWidget *MicroBlog::graphicsWidget()
     m_serviceUrl = cg.readEntry("serviceUrl", "https://identi.ca/api/");
     m_username = cg.readEntry("username");
     m_password = KStringHandler::obscure(cg.readEntry("password"));
-    m_historySize = cg.readEntry("historySize", 2);
+    m_historySize = cg.readEntry("historySize", 6);
     m_historyRefresh = cg.readEntry("historyRefresh", 5);
     m_includeFriends = cg.readEntry("includeFriends", true);
 
@@ -191,7 +183,6 @@ QGraphicsWidget *MicroBlog::graphicsWidget()
 
     m_flash->setAutohide( true );
     m_flash->setMinimumSize( 0, 20 );
-    m_flash->setColor( Plasma::Theme::TextColor );
     QFont fnt = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
     fnt.setBold( true );
     QFontMetrics fm( fnt );
@@ -210,28 +201,26 @@ QGraphicsWidget *MicroBlog::graphicsWidget()
 
     m_layout->addItem(flashLayout);
 
-    Plasma::Frame *headerFrame = new Plasma::Frame(this);
-    m_headerLayout = new QGraphicsLinearLayout( Qt::Horizontal, headerFrame );
-    m_headerLayout->setContentsMargins( 5, 5, 5, 10 );
+    m_headerFrame = new Plasma::Frame(this);
+    m_headerLayout = new QGraphicsAnchorLayout(m_headerFrame);
     m_headerLayout->setSpacing( 5 );
-    m_layout->addItem( headerFrame );
 
-
-    m_icon = new Plasma::IconWidget(this);
+    m_icon = new Plasma::IconWidget(m_headerFrame);
     m_icon->setIcon(KIcon("user-identity"));
     m_icon->setText(m_username);
     m_icon->setTextBackgroundColor(QColor());
     QSizeF iconSize = m_icon->sizeFromIconSize(48);
     m_icon->setMinimumSize( iconSize );
     m_icon->setMaximumSize( iconSize );
-    m_headerLayout->addItem( m_icon );
+    m_headerLayout->addAnchor(m_icon, Qt::AnchorVerticalCenter, m_headerLayout, Qt::AnchorVerticalCenter);
+    m_headerLayout->addAnchor(m_icon, Qt::AnchorLeft, m_headerLayout, Qt::AnchorLeft);
 
-    Plasma::Frame *statusEditFrame = new Plasma::Frame(this);
+    Plasma::Frame *statusEditFrame = new Plasma::Frame(m_headerFrame);
 
-    statusEditFrame->setMaximumHeight(fm.height() * 4);
     statusEditFrame->setFrameShadow(Plasma::Frame::Sunken);
     QGraphicsLinearLayout *statusEditLayout = new QGraphicsLinearLayout(statusEditFrame);
     m_statusEdit = new Plasma::TextEdit(this);
+    m_statusEdit->setPreferredHeight(fm.height() * 4);
 
     connect(m_statusEdit, SIGNAL(textChanged()), this, SLOT(editTextChanged()));
     statusEditLayout->addItem(m_statusEdit);
@@ -242,12 +231,25 @@ QGraphicsWidget *MicroBlog::graphicsWidget()
     editPal.setColor(QPalette::Text, m_colorScheme->foreground().color());
     m_statusEdit->nativeWidget()->setPalette(editPal);
     m_statusEdit->nativeWidget()->installEventFilter(this);
-    m_headerLayout->addItem( statusEditFrame );
+    m_headerLayout->addAnchor(m_icon, Qt::AnchorRight, statusEditFrame, Qt::AnchorLeft);
+    m_headerLayout->addAnchors(statusEditFrame, m_headerLayout, Qt::Vertical);
+    m_headerLayout->addAnchor(statusEditFrame, Qt::AnchorRight, m_headerLayout, Qt::AnchorRight);
+    m_headerLayout->activate();
+    m_headerLayout->setMaximumHeight(m_headerLayout->effectiveSizeHint(Qt::PreferredSize).height());
+
+    m_tabBar = new Plasma::TabBar(this);
+    m_tabBar->addTab(i18n("Timeline"));
+    m_tabBar->addTab(i18n("Replies"));
+    m_tabBar->addTab(i18n("Messages"));
+    m_layout->addItem(m_tabBar);
+    connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(modeChanged(int)));
+    m_tabBar->nativeWidget()->installEventFilter(this);
 
     m_scrollWidget = new Plasma::ScrollWidget(this);
-    m_tweetsWidget = new QGraphicsWidget;
+    m_tweetsWidget = new QGraphicsWidget(m_scrollWidget);
     m_scrollWidget->setWidget(m_tweetsWidget);
     m_tweetsLayout = new QGraphicsLinearLayout(Qt::Vertical, m_tweetsWidget);
+    m_tweetsLayout->addItem(m_headerFrame);
 
     m_layout->addItem(m_scrollWidget);
 
@@ -272,6 +274,30 @@ QGraphicsWidget *MicroBlog::graphicsWidget()
     return m_graphicsWidget;
 }
 
+void MicroBlog::modeChanged(int index)
+{
+    m_tweetMap.clear();
+    m_lastTweet=0;
+    downloadHistory();
+}
+
+void MicroBlog::reply(const QString &to)
+{
+    m_scrollWidget->ensureItemVisible(m_headerFrame);
+    m_statusEdit->nativeWidget()->setPlainText(to);
+    QTextCursor cursor = m_statusEdit->nativeWidget()->textCursor();
+    cursor.movePosition(QTextCursor::End);
+    m_statusEdit->nativeWidget()->setTextCursor(cursor);
+    m_statusEdit->nativeWidget()->setFocus();
+}
+
+void MicroBlog::forward(const QString &rt)
+{
+    m_scrollWidget->ensureItemVisible(m_headerFrame);
+    m_statusEdit->nativeWidget()->setPlainText(rt);
+    m_statusEdit->nativeWidget()->setFocus();
+}
+
 void MicroBlog::getWallet()
 {
     //TODO: maybe Plasma in general should handle the wallet
@@ -383,6 +409,7 @@ void MicroBlog::dataUpdated(const QString& source, const Plasma::DataEngine::Dat
         int newCount = 0;
         qulonglong maxId = m_lastTweet;
         foreach (const QString &id, data.keys()) {
+
             qulonglong i = id.toULongLong();
             //kDebug() << i << m_lastTweet;
             if (i > m_lastTweet) {
@@ -391,6 +418,7 @@ void MicroBlog::dataUpdated(const QString& source, const Plasma::DataEngine::Dat
                 //Warning: This function is not available with MSVC 6
                 Plasma::DataEngine::Data t = v.value<Plasma::DataEngine::Data>();
                 m_tweetMap[i] = t;
+
                 if (i > maxId) {
                     maxId = i;
                 }
@@ -418,9 +446,16 @@ void MicroBlog::dataUpdated(const QString& source, const Plasma::DataEngine::Dat
                     m_icon->setAction(profile);
                     m_icon->setMinimumSize( iconSize );
                     m_icon->setMaximumSize( iconSize );
-                    connect(profile, SIGNAL(triggered(bool)), this, SLOT(openProfile()));
+                    connect(profile, SIGNAL(triggered()), this, SLOT(openProfile()));
                 }
                 m_pictureMap[user] = pm;
+                m_avatarHistory.removeAll(user);
+                m_avatarHistory.append(user);
+                if (m_avatarHistory.size() > 30) {
+                    QString oldestUser = m_avatarHistory.first();
+                    m_avatarHistory.pop_front();
+                    m_pictureMap.remove(oldestUser);
+                }
                 //TODO it would be nice to check whether the updated image is actually in use
                 scheduleShowTweets();
             }
@@ -434,7 +469,8 @@ void MicroBlog::dataUpdated(const QString& source, const Plasma::DataEngine::Dat
 
         m_flash->flash(desc, 60 * 1000); //I'd really prefer it to stay there. and be red.
     }
-    //updateGeometry();
+    m_graphicsWidget->setPreferredSize(-1, -1);
+    emit sizeHintChanged(Qt::PreferredSize);
 }
 
 void MicroBlog::themeChanged()
@@ -471,106 +507,41 @@ void MicroBlog::showTweets()
     }
 
     while (m_tweetWidgets.count() < m_tweetMap.count()) {
-        Plasma::Frame *tweetFrame = new Plasma::Frame(m_tweetsWidget);
-        tweetFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-
-        QGraphicsLinearLayout *tweetLayout = new QGraphicsLinearLayout( Qt::Horizontal, tweetFrame );
-        tweetLayout->setContentsMargins( 0, 5, 0, 5 );
-        tweetLayout->setSpacing( 5 );
-        //m_layout->insertItem( m_layout->count()-1, tweetFrame );
-        m_tweetsLayout->addItem(tweetFrame);
-
-        Plasma::TextBrowser *tweetText = new Plasma::TextBrowser(tweetFrame);
-        tweetText->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-        tweetText->nativeWidget()->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-        tweetText->nativeWidget()->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-        tweetText->nativeWidget()->setCursor( Qt::ArrowCursor );
-
-        Plasma::IconWidget *icon = new Plasma::IconWidget(tweetFrame);
-        QSizeF iconSize = icon->sizeFromIconSize(30);
-        icon->setMinimumSize( iconSize );
-        icon->setMaximumSize( iconSize );
-
-        Plasma::IconWidget *favIcon = new Plasma::IconWidget(tweetFrame);
-        QSizeF favIconSize = icon->sizeFromIconSize(16);
-        favIcon->setMinimumSize( favIconSize );
-        favIcon->setMaximumSize( favIconSize );
-
-        tweetLayout->addItem(icon);
-        tweetLayout->addItem(tweetText);
-        tweetLayout->addItem(favIcon);
-
-        Tweet t;
-        t.frame = tweetFrame;
-        t.icon = icon;
-        t.content = tweetText;
-        t.favIcon = favIcon;
-
-        m_tweetWidgets.append( t );
+        PostWidget *postWidget = new PostWidget(m_tweetsWidget);
+        connect(postWidget, SIGNAL(reply(const QString &)), this, SLOT(reply(const QString &)));
+        connect(postWidget, SIGNAL(forward(const QString &)), this, SLOT(forward(const QString &)));
+        connect(postWidget, SIGNAL(openProfile(const QString &)), this, SLOT(openProfile(const QString &)));
+        m_tweetWidgets.append(postWidget);
     }
 
     //clear out tweet widgets if there are too many
     while (m_tweetWidgets.count() > m_tweetMap.count()) {
-        Tweet t = m_tweetWidgets[m_tweetWidgets.size() - 1];
-        m_layout->removeItem(t.frame);
-        QAction *iconAction = t.icon->action();
-        t.icon->setAction(0);
-        delete iconAction;
-        delete t.frame;
+        PostWidget *t = m_tweetWidgets[m_tweetWidgets.size() - 1];
+        m_layout->removeItem(t);
         m_tweetWidgets.removeAt(m_tweetWidgets.size() - 1);
+        t->deleteLater();
+        m_tweetsWidget->resize(m_tweetsWidget->effectiveSizeHint(Qt::PreferredSize));
     }
 
     int i = 0;
     QMap<qulonglong, Plasma::DataEngine::Data>::iterator it = m_tweetMap.end();
+
     while (it != m_tweetMap.begin()) {
         Plasma::DataEngine::Data &tweetData = *(--it);
+
         QString user = tweetData.value("User").toString();
-        QPixmap favIcon = tweetData.value("SourceFavIcon").value<QPixmap>();
 
         if (i == 0) {
             m_popupIcon = m_pictureMap[user];
         }
 
-        QAction *profile = new QAction(QIcon(m_pictureMap[user]), QString(), this);
-        profile->setData(user);
-
-        Tweet t = m_tweetWidgets[i];
-        t.icon->setAction(profile);
-        QSizeF iconSize = t.icon->sizeFromIconSize(30);
-        t.icon->setMinimumSize(iconSize);
-        t.icon->setMaximumSize(iconSize);
-        connect(profile, SIGNAL(triggered(bool)), this, SLOT(openProfile()));
-
-        QString sourceString;
-
-        if (favIcon.isNull()) {
-            sourceString = i18n(" from %1", tweetData.value( "Source" ).toString());
-        }
-
-        QLocale english(QLocale::English, QLocale::UnitedStates);
-        QDateTime dt = english.toDateTime(tweetData.value( "Date" ).toString(), "ddd MMM dd HH:mm:ss +0000 yyyy");
-        dt.setTimeSpec(Qt::UTC);
-        QString html = "<table cellspacing='0' spacing='5' width='100%'>";
-        html += QString("<tr height='1em'><td align='left' width='1%'><font color='%2'>%1</font></td><td align='right' width='auto'><p align='right'><font color='%2'>%3%4</font></p></td></tr></table>").arg( user).arg(m_colorScheme->foreground(KColorScheme::InactiveText).color().name())
-                .arg(timeDescription( dt )).arg( sourceString);
-        QString status = tweetData.value( "Status" ).toString();
-
-        status.replace(QRegExp("((http|https)://[^\\s<>'\"]+[^!,\\.\\s<>'\"\\]])"), "<a href='\\1'>\\1</a>");
-
-        html += QString( "<p><font color='%1'>%2</font></p>" )
-                .arg( m_colorScheme->foreground().color().name()).arg( status );
-
-        t.content->setText( html );
-        t.content->nativeWidget()->document()->setDefaultStyleSheet(QString("a{color:%1} a:visited{color:%2}")
-                                            .arg( m_colorScheme->foreground(KColorScheme::LinkText).color().name())
-                                            .arg(m_colorScheme->foreground(KColorScheme::VisitedText).color().name()));
-
-
-        t.content->update();
-
-        if( !favIcon.isNull() ) {
-            t.favIcon->setIcon( QIcon(favIcon) );
-        }
+        PostWidget *t = m_tweetWidgets[i];
+        t->setColorScheme(m_colorScheme);
+        t->setData(tweetData);
+        t->setPicture(m_pictureMap[user]);
+        //you don't want to reply or RT yourself
+        t->setActionsShown(user != m_username);
+        m_tweetsLayout->addItem(t);
 
         ++i;
     }
@@ -589,28 +560,20 @@ void MicroBlog::createConfigurationInterface(KConfigDialog *parent)
     QWidget *configWidget = new QWidget();
     configUi.setupUi(configWidget);
 
-    connect(configUi.historySizeSpinBox, SIGNAL(valueChanged(int)), this, SLOT(updateSpinBoxSuffix()));
-    connect(configUi.historyRefreshSpinBox, SIGNAL(valueChanged(int)), this, SLOT(updateSpinBoxSuffix()));
-
     configUi.serviceUrlCombo->addItem("https://identi.ca/api/");
     configUi.serviceUrlCombo->addItem("https://twitter.com/");
     configUi.serviceUrlCombo->setEditText(m_serviceUrl);
     configUi.usernameEdit->setText(m_username);
     configUi.passwordEdit->setText(m_password);
     configUi.historySizeSpinBox->setValue(m_historySize);
+    configUi.historySizeSpinBox->setSuffix(ki18np(" message", " messages"));
     configUi.historyRefreshSpinBox->setValue(m_historyRefresh);
+    configUi.historyRefreshSpinBox->setSuffix(ki18np(" minute", " minutes"));
     configUi.checkIncludeFriends->setChecked(m_includeFriends);
-    updateSpinBoxSuffix();
 
     parent->addPage(configWidget, i18n("General"), icon());
 }
 
-void MicroBlog::updateSpinBoxSuffix()
-{
-    configUi.historySizeSpinBox->setSuffix(i18np(" message", " messages", configUi.historySizeSpinBox->value()));
-    configUi.historyRefreshSpinBox->setSuffix(i18np(" minute", " minutes", configUi.historyRefreshSpinBox->value()));
-}
-
 void MicroBlog::configAccepted()
 {
     QString serviceUrl = configUi.serviceUrlCombo->currentText();
@@ -694,6 +657,16 @@ void MicroBlog::configAccepted()
 
     //kDebug() << changed << reloadRequired;
     if (changed) {
+        if (m_service) {
+            m_service->deleteLater();
+            m_service = 0;
+        }
+
+        if (m_profileService) {
+            m_profileService->deleteLater();
+            m_profileService = 0;
+        }
+
         if (reloadRequired) {
             m_lastTweet = 0;
             downloadHistory();
@@ -710,7 +683,6 @@ MicroBlog::~MicroBlog()
     delete m_colorScheme;
     delete m_service;
     delete m_profileService;
-    delete m_statusService;
 }
 
 void MicroBlog::editTextChanged()
@@ -737,6 +709,10 @@ bool MicroBlog::eventFilter(QObject *obj, QEvent *event)
         } else {
             return false;
         }
+    } else if (obj == m_tabBar->nativeWidget() && event->type() == QEvent::MouseButtonPress) {
+        m_scrollWidget->ensureItemVisible(m_headerFrame);
+        m_statusEdit->setFocus();
+        return false;
     } else {
         return Plasma::Applet::eventFilter(obj, event);
     }
@@ -744,25 +720,32 @@ bool MicroBlog::eventFilter(QObject *obj, QEvent *event)
 
 void MicroBlog::updateStatus()
 {
+    createTimelineService();
     QString status = m_statusEdit->nativeWidget()->toPlainText();
 
-    if (!m_statusService) {
-        m_statusService = m_engine->serviceForSource(m_curTimeline);
-    }
-
     KConfigGroup cg = m_service->operationDescription("update");
     cg.writeEntry("password", m_password);
     cg.writeEntry("status", status);
-    m_service->startOperationCall(cg);
-    //m_statusUpdates.insert(m_service->startOperationCall(cg), status);
-    connect(m_service, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(updateCompleted(Plasma::ServiceJob*)));
-    connect(m_service, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(serviceFinished(Plasma::ServiceJob*)));
 
+    if (m_updateJobs.isEmpty()) {
+        connect(m_service, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(updateCompleted(Plasma::ServiceJob*)));
+    }
+
+    m_updateJobs.insert(m_service->startOperationCall(cg));
     m_statusEdit->nativeWidget()->setPlainText("");
 }
 
 void MicroBlog::updateCompleted(Plasma::ServiceJob *job)
 {
+    if (!m_updateJobs.contains(job)) {
+        return;
+    }
+
+    m_updateJobs.remove(job);
+    if (m_updateJobs.isEmpty()) {
+        disconnect(m_service, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(updateCompleted(Plasma::ServiceJob*)));
+    }
+
     if (!job->error()) {
         //m_statusUpdates.value(job);
         downloadHistory();
@@ -785,92 +768,108 @@ void MicroBlog::downloadHistory()
 
     m_flash->flash(i18n("Refreshing timeline..."), -1);
 
-    QString query;
-    if (m_includeFriends) {
-        query = QString("TimelineWithFriends:%1@%2");
+    if (m_service && m_tabBar->currentIndex() == m_lastMode) {
+        KConfigGroup cg = m_service->operationDescription("refresh");
+        m_service->startOperationCall(cg);
     } else {
-        query = QString("Timeline:%1@%2");
+        createTimelineService();
+        KConfigGroup cg = m_service->operationDescription("auth");
+        cg.writeEntry("password", m_password);
+        m_service->startOperationCall(cg);
+    }
+
+    //get the profile to retrieve the user icon
+
+    if (m_profileService) {
+        KConfigGroup cg = m_profileService->operationDescription("refresh");
+        m_profileService->startOperationCall(cg);
+    } else {
+        QString profileQuery(QString("Profile:%1@%2").arg(m_username, m_serviceUrl));
+        m_engine->connectSource(m_imageQuery, this);
+        m_engine->connectSource(profileQuery, this, m_historyRefresh * 60 * 1000);
+        m_profileService = m_engine->serviceForSource(profileQuery);
+        connect(m_profileService, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(serviceFinished(Plasma::ServiceJob*)));
+        KConfigGroup profileConf = m_profileService->operationDescription("auth");
+        profileConf.writeEntry("password", m_password);
+        m_profileService->startOperationCall(profileConf);
+    }
+}
+
+void MicroBlog::createTimelineService()
+{
+    if (m_service && m_lastMode == m_tabBar->currentIndex()) {
+        return;
+    } else if (m_service) {
+        delete m_service;
+        m_lastMode = m_tabBar->currentIndex();
+    }
+
+    QString query;
+    switch(m_tabBar->currentIndex()) {
+    case 2:
+        query = "Messages:%1@%2";
+        break;
+    case 1:
+        query = "Replies:%1@%2";
+        break;
+    default:
+        if(m_includeFriends) {
+            query = QString("TimelineWithFriends:%1@%2");
+        } else {
+            query = QString("Timeline:%1@%2");
+        }
+        break;
     }
 
     query = query.arg(m_username, m_serviceUrl);
     //kDebug() << m_curTimeline << query;
-    bool newQuery = m_curTimeline != query;
-    if (newQuery) {
+    if (m_curTimeline != query) {
         //ditch the old one, if needed
         if (!m_curTimeline.isEmpty()) {
             m_engine->disconnectSource(m_curTimeline, this);
             m_engine->disconnectSource("Error:" + m_curTimeline, this);
         }
 
-        delete m_statusService;
-        m_statusService = 0;
-
         m_curTimeline = query;
     }
 
     //kDebug() << "Connecting to source " << query << "with refresh rate" << m_historyRefresh * 60 * 1000;
     m_engine->connectSource(query, this, m_historyRefresh * 60 * 1000);
     m_engine->connectSource("Error:" + query, this);
-
-    if (m_service) {
-        m_service->deleteLater();
-    }
     m_service = m_engine->serviceForSource(m_curTimeline);
     connect(m_service, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(serviceFinished(Plasma::ServiceJob*)));
-    KConfigGroup cg = m_service->operationDescription("auth");
-    cg.writeEntry("password", m_password);
-    m_service->startOperationCall(cg);
-
-    //get the profile to retrieve the user icon
-    QString profileQuery(QString("Profile:%1@%2").arg(m_username, m_serviceUrl));
-
-    m_engine->connectSource(m_imageQuery, this);
-    m_engine->connectSource(profileQuery, this, m_historyRefresh * 60 * 1000);
-
-    if (m_profileService) {
-        m_profileService->deleteLater();
-    }
-    m_profileService = m_engine->serviceForSource(profileQuery);
-    connect(m_profileService, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(serviceFinished(Plasma::ServiceJob*)));
-    KConfigGroup profileConf = m_profileService->operationDescription("auth");
-    profileConf.writeEntry("password", m_password);
-    m_profileService->startOperationCall(profileConf);
 }
 
 void MicroBlog::serviceFinished(Plasma::ServiceJob *job)
 {
     if (job->error()) {
         m_flash->flash(job->errorString(), 2000);
+
+        // reset the service objects to give it a chance
+        // to re-authenticate
+        if (m_service) {
+            m_service->deleteLater();
+            m_service = 0;
+        }
+
+        if (m_profileService) {
+            m_profileService->deleteLater();
+            m_profileService = 0;
+        }
     }
 }
 
-void MicroBlog::openProfile()
+void MicroBlog::openProfile(const QString &profile)
 {
-    QAction *action = qobject_cast<QAction *>(sender());
-
-    if (action) {
-        QString url = m_serviceUrl;
-        url.remove("api/");
-
-        KRun::runUrl( KUrl(KUrl(url), action->data().toString()), "text/html", 0 );
+    QString url = m_serviceUrl;
+    url.remove("api/");
+    if (!profile.isEmpty()) {
+        KToolInvocation::invokeBrowser(KUrl(KUrl(url), profile).prettyUrl());
+    } else {
+        KToolInvocation::invokeBrowser(KUrl(KUrl(url), m_username).prettyUrl());
     }
 }
 
-QString MicroBlog::timeDescription( const QDateTime &dt )
-{
-    int diff = dt.secsTo(KDateTime::currentDateTime(m_tz).dateTime());
 
-    if (diff < 60) {
-        return i18n("Less than a minute ago");
-    } else if (diff < 60*60) {
-        return i18np("1 minute ago", "%1 minutes ago", diff/60);
-    } else if (diff < 2*60*60) {
-        return i18n("Over an hour ago");
-    } else if (diff < 24*60*60) {
-        return i18np("1 hour ago", "%1 hours ago", diff/3600);
-    }
-
-    return dt.toString(Qt::LocaleDate);
-}
 
 #include "microblog.moc"
