@@ -17,6 +17,8 @@
     Boston, MA 02110-1301, USA.
 */
 
+#include "contactwidget.h"
+
 //Qt
 #include <QGraphicsGridLayout>
 #include <QGraphicsSceneMouseEvent>
@@ -28,7 +30,7 @@
 #include <KIcon>
 #include <KIconLoader>
 #include <KGlobalSettings>
-#include <KRun>
+#include <KToolInvocation>
 
 // Plasma
 #include <Plasma/Theme>
@@ -37,21 +39,21 @@
 #include <Plasma/Frame>
 
 // own
-#include "contactwidget.h"
 #include "contactimage.h"
+#include "utils.h"
 
 using namespace Plasma;
 
-ContactWidget::ContactWidget(QGraphicsWidget *parent)
+ContactWidget::ContactWidget(DataEngine* engine, QGraphicsWidget* parent)
     : Frame(parent),
-      id(0),
       m_isHovered(false),
       m_isFriend(false),
       m_image(0),
       m_nameLabel(0),
       m_sendMessage(0),
       m_addFriend(0),
-      m_showDetails(0)
+      m_showDetails(0),
+      m_engine(engine)
 {
     setAcceptHoverEvents(true);
     buildDialog();
@@ -66,21 +68,59 @@ ContactWidget::~ContactWidget()
 {
 }
 
-void ContactWidget::setAtticaData(const Plasma::DataEngine::Data &data)
-{
-    //kDebug() << data;
-    m_ocsData = data;
-    QString _id = data["Id"].toString();
 
-    QString name = data["Name"].toString();
+void ContactWidget::setId(const QString& id)
+{
+    if (!m_provider.isEmpty() && !m_id.isEmpty()) {
+        m_engine->disconnectSource(personSummaryQuery(m_provider, m_id), this);
+    }
+    m_id = id;
+    if (!m_provider.isEmpty() && !m_id.isEmpty()) {
+        m_engine->connectSource(personSummaryQuery(m_provider, m_id), this);
+    }
+}
+
+
+QString ContactWidget::id() const
+{
+    return m_id;
+}
+
+
+void ContactWidget::setProvider(const QString& provider)
+{
+    if (!m_provider.isEmpty() && !m_id.isEmpty()) {
+        m_engine->disconnectSource(personSummaryQuery(m_provider, m_id), this);
+    }
+    m_provider = provider;
+    if (!m_provider.isEmpty() && !m_id.isEmpty()) {
+        m_engine->connectSource(personSummaryQuery(m_provider, m_id), this);
+    }
+}
+
+
+QString ContactWidget::provider() const
+{
+    return m_provider;
+}
+
+
+void ContactWidget::dataUpdated(const QString& source, const Plasma::DataEngine::Data& data)
+{
+    Q_UNUSED(source);
+    
+    m_ocsData = data.value(personAddPrefix(m_id)).value<DataEngine::Data>();
+    QString _id = m_ocsData["Id"].toString();
+
+    QString name = m_ocsData["Name"].toString();
     if (name.isEmpty()) {
         setName(_id);
     } else {
         setName(QString("%1 (%2)").arg(name, _id));
     }
 
-    QString city = data["City"].toString();
-    QString country = data["Country"].toString();
+    QString city = m_ocsData["City"].toString();
+    QString country = m_ocsData["Country"].toString();
     QString location;
     if (!city.isEmpty() && !country.isEmpty()) {
         location = QString("%1, %2").arg(city, country);
@@ -93,15 +133,10 @@ void ContactWidget::setAtticaData(const Plasma::DataEngine::Data &data)
     if (!location.isEmpty()) {
         setInfo(location);
     }
-    QPixmap pm = data["Avatar"].value<QPixmap>();
-    QString pmUrl = data["AvatarUrl"].toUrl().toString();
-    if (!pm.isNull()) {
-        m_image->setPixmap(pm);
-    } else {
-        kDebug() << "avatarUrl" << pmUrl;
-    }
+    m_image->setUrl(m_ocsData.value("AvatarUrl").toUrl());
 }
 
+
 void ContactWidget::buildDialog()
 {
     updateColors();
@@ -113,7 +148,7 @@ void ContactWidget::buildDialog()
     m_layout->setColumnFixedWidth(0, int(m*1.2));
     m_layout->setHorizontalSpacing(4);
 
-    m_image = new ContactImage(this);
+    m_image = new ContactImage(m_engine, this);
     m_image->setMinimumHeight(m);
     m_image->setMinimumWidth(m);
     m_layout->addItem(m_image, 0, 0, 2, 1, Qt::AlignTop);
@@ -159,9 +194,9 @@ void ContactWidget::buildDialog()
     m_showDetails->setMinimumWidth(s);
     m_showDetails->setMaximumWidth(s);
 
-    connect(m_sendMessage, SIGNAL(clicked()), this, SLOT(sendMessage()));
-    connect(m_addFriend, SIGNAL(clicked()), this, SLOT(addFriend()));
-    connect(m_showDetails, SIGNAL(clicked()), this, SLOT(userProperties()));
+    connect(m_sendMessage, SIGNAL(clicked()), SIGNAL(sendMessage()));
+    connect(m_addFriend, SIGNAL(clicked()), SIGNAL(addFriend()));
+    connect(m_showDetails, SIGNAL(clicked()), SLOT(slotShowDetails()));
 
     m_actions->addItem(m_addFriend);
     m_actions->addItem(m_sendMessage);
@@ -177,8 +212,10 @@ void ContactWidget::buildDialog()
 
 void ContactWidget::setIsFriend(bool isFriend)
 {
-    m_isFriend = isFriend;
-    updateActions();
+    if (m_isFriend != isFriend) {
+        m_isFriend = isFriend;
+        updateActions();
+    }
 }
 
 void ContactWidget::updateActions()
@@ -188,32 +225,15 @@ void ContactWidget::updateActions()
     m_showDetails->setVisible(m_isHovered);
 }
 
-void ContactWidget::setImage(const QImage &image)
-{
-    m_image->setImage(image);
-}
 
-void ContactWidget::sendMessage()
-{
-    QUrl url = QString("https://www.opendesktop.org/messages/?action=newmessage&username=%1").arg(user());
-    new KRun(url, m_nameLabel->nativeWidget());
-}
-
-void ContactWidget::addFriend()
-{
-    KUrl url = QString("https://www.opendesktop.org/usermanager/relationadd.php?username=%1").arg(user());
-    kDebug() << "add friend" << user() << url.url();
-    new KRun(url, m_nameLabel->nativeWidget());
-}
-
-void ContactWidget::userProperties()
+void ContactWidget::slotShowDetails()
 {
     kDebug() << "user details" << user();
     // When sliding out, we'll usually miss the hoverLeaveEvent,
     // so do it manually
     m_isHovered = false;
     updateActions();
-    emit showDetails(m_ocsData);
+    emit showDetails();
 }
 
 void ContactWidget::updateColors()
@@ -236,10 +256,11 @@ void ContactWidget::updateColors()
 
     setPalette(p);
 
-    if (m_image) {
+    // FIXME: Re-activate or delete
+    /*if (m_image) {
         m_image->fg = text;
         m_image->bg = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor);
-    }
+    }*/
 
     qreal fontsize = KGlobalSettings::smallestReadableFont().pointSize();
     m_stylesheet = QString("\
@@ -267,22 +288,18 @@ void ContactWidget::updateColors()
     }
 }
 
-Plasma::DataEngine::Data ContactWidget::data()
-{
-    return m_ocsData;
-}
 
 void ContactWidget::setName(const QString &name)
 {
     m_nameLabel->setText(name);
 }
 
-QString ContactWidget::name()
+QString ContactWidget::name() const
 {
     return m_ocsData["Name"].toString();
 }
 
-QString ContactWidget::user()
+QString ContactWidget::user() const
 {
     return m_ocsData["Id"].toString();
 }
@@ -312,4 +329,11 @@ void ContactWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
     updateActions();
 }
 
+
+void ContactWidget::mousePressEvent(QGraphicsSceneMouseEvent* event)
+{
+    emit showDetails();
+}
+
+
 #include "contactwidget.moc"
