@@ -23,16 +23,14 @@
 #include <QGraphicsSceneWheelEvent>
 #include <QPainter>
 #include <QStyleOptionGraphicsItem>
-#include <QTimeLine>
 
 // KDE
 #include <KDebug>
-#include <KConfigGroup>
 #include <KGlobalSettings>
-#include <KService>
 #include <KIconLoader>
 #include <KIO/PreviewJob>
 #include <KUrl>
+#include <KIcon>
 
 // Plasma
 #include <Plasma/Animator>
@@ -41,12 +39,14 @@
 #include <Plasma/Svg>
 #include <Plasma/Theme>
 
-const int EXPANDING_DURATION = 200;
+static const int EXPANDING_DURATION = 200;
+static const int REMOVE_EMBLEM_SIZE = 22;
 
 PreviewWidget::PreviewWidget(QGraphicsItem *parent)
     : QGraphicsWidget(parent),
       m_selectedIndex(-1),
       m_hoveredIndex(-1),
+      m_hoverSvg(new Plasma::FrameSvg(this)),
       m_closeStatus(true),
       m_animId(-1)
 {
@@ -63,13 +63,21 @@ PreviewWidget::PreviewWidget(QGraphicsItem *parent)
 
     // let's draw the top border of the applet
     m_panel = new Plasma::FrameSvg(this);
-    m_panel->setImagePath("dialogs/background");
+    m_panel->setImagePath("widgets/background");
 
     m_logo = new Plasma::Svg(this);
     m_logo->setImagePath("widgets/previewer-16");
+    
+    m_hoverSvg->setImagePath("widgets/viewitem");
+    m_hoverSvg->setEnabledBorders(Plasma::FrameSvg::AllBorders);
+    m_hoverSvg->setCacheAllRenderedFrames(true);
+
+    updateMargins();
 
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
             this, SLOT(setupOptionViewItem()));
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
+            this, SLOT(updateMargins()));
 }
 
 PreviewWidget::~PreviewWidget()
@@ -149,7 +157,6 @@ void PreviewWidget::wheelEvent(QGraphicsSceneWheelEvent *event)
 
 void PreviewWidget::lookForPreview()
 {
-    m_previewReady = false;
     if (m_previewHistory.isEmpty()) {
         return;
     }
@@ -168,18 +175,7 @@ void PreviewWidget::lookForPreview()
 void PreviewWidget::setPreview(const KFileItem &item, const QPixmap &pixmap)
 {
     m_previews.insert(item.url(), pixmap);
-    m_previewReady = true;
-    updatePreview();
-}
-
-void PreviewWidget::updatePreview()
-{
-    //NOTE: use this to update the exact rect in order to repaint
-    //      correctly the preview
-
-    QRect rect = m_previewRect;
-    rect.setHeight(rect.height() + bottomBorderHeight());
-    update(rect);
+    update();
 }
 
 void PreviewWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -211,10 +207,24 @@ void PreviewWidget::updateSelectedItems(const QPoint &point)
     for (int i = 0; i < m_items.count(); i++) {
         if (m_items[i].contains(point)) {
             m_selectedIndex = i;
-            emit fileOpenRequested(KUrl(m_previewHistory[i]));
             break;
         }
     }
+    
+    if (m_selectedIndex == -1) {
+        return;
+    }
+
+    // here we check if the remove emblem was clicked
+    QRect r = m_items[m_selectedIndex];
+    r.setX(r.right() - REMOVE_EMBLEM_SIZE);
+    r.setSize(QSize(REMOVE_EMBLEM_SIZE, REMOVE_EMBLEM_SIZE));
+    if (r.contains(point)) {
+        removeItem(m_selectedIndex);
+        return;
+    }
+
+    emit fileOpenRequested(KUrl(m_previewHistory[m_selectedIndex]));
 
     if (m_selectedIndex != previous) {
         if (m_selectedIndex != -1) {
@@ -229,6 +239,12 @@ void PreviewWidget::updateSelectedItems(const QPoint &point)
 
 void PreviewWidget::updateHoveredItems(const QPoint &point)
 {
+
+    // updating is unsafe when layout is broken
+    if (!m_layoutIsValid) {
+        return;
+    }
+
     m_hoveredUrl = KUrl();
     const int previous = m_hoveredIndex;
     m_hoveredIndex = -1;
@@ -243,7 +259,7 @@ void PreviewWidget::updateHoveredItems(const QPoint &point)
                 KUrl item = KUrl(m_previewHistory[i]);
                 if (m_previews.contains(item)) {
                     m_hoveredUrl = item;
-                    updatePreview();
+                    update();
                 }
 
                 break;
@@ -279,9 +295,7 @@ void PreviewWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 
         if (m_itemsRect.contains(event->pos().toPoint())) {
             updateSelectedItems(event->pos().toPoint());
-        }
-
-        if (m_arrowRect.contains(event->pos().toPoint())) {
+        } else if (m_arrowRect.contains(event->pos().toPoint())) {
             if (m_closeStatus) {
                 expand();
             } else {
@@ -308,7 +322,16 @@ void PreviewWidget::setItemsList(const QList<QUrl> &list)
     m_previewHistory.clear();
     m_previewHistory = list;
 
-    calculateRects();
+    m_layoutIsValid = false;
+    update();
+
+    lookForPreview();
+}
+
+void PreviewWidget::addItem(const QUrl &url)
+{
+    m_previewHistory << url;
+    m_layoutIsValid = false;
     update();
 
     lookForPreview();
@@ -322,7 +345,7 @@ void PreviewWidget::resizeEvent(QGraphicsSceneResizeEvent *event)
 
 void PreviewWidget::calculateRects()
 {
-    QRect rect = contentsRect().toRect();
+    QRect rect = this->rect().toRect();
     const int scrollBarWidth = m_scrollBar->isVisible() ? int(m_scrollBar->preferredSize().width()) : 0;
     const int itemWidth = iconSize().width() * 2;
     const int itemRectWidth = itemWidth + scrollBarWidth + (s_spacing * 2) +
@@ -332,22 +355,25 @@ void PreviewWidget::calculateRects()
     // items rect shouldn't be visible even if the status is close
     if (m_previewHistory.isEmpty() || (m_closeStatus && m_animId < 1)) {
         m_animationHeight = s_topBorder + bottomBorder;
-        m_scrollBar->setGeometry(QRect());
     } else {
         if (m_animId < 1) {
             m_animationHeight = rect.height();
         }
-        const int itemRectHeight = m_animationHeight - s_topBorder - bottomBorder;
-        m_itemsRect = QRect(5, s_topBorder, itemRectWidth, itemRectHeight);
+    }
 
+    const int itemRectHeight = m_animationHeight - s_topBorder - bottomBorder;
+    m_itemsRect = QRect(contentsRect().x(), s_topBorder, itemRectWidth, itemRectHeight);
+
+    if (itemRectHeight) {
         QRect r = QRect(m_itemsRect.right() - scrollBarWidth + 2, s_topBorder + 1, scrollBarWidth,
                         m_animationHeight - s_topBorder - bottomBorder - 2);
         m_scrollBar->setGeometry(r);
+    } else {
+        m_scrollBar->setGeometry(QRect());
     }
 
     //kDebug() << m_animationHeight;
-
-    m_arrowRect = QRect(rect.width() - 30 - 5, (s_topBorder / 2), 10,10);
+    m_arrowRect = QRect(contentsRect().right() - 10, (s_topBorder / 2), 10,10);
 
     m_previewRect = QRect(m_itemsRect.right(), m_itemsRect.y(),
                          rect.width() - m_itemsRect.right() - rect.x(),
@@ -466,8 +492,8 @@ void PreviewWidget::paint(QPainter *painter,
     }
 
     const int bottomBorder = bottomBorderHeight();
-    QRect contentsRect = this->contentsRect().toRect();
-    painter->translate(contentsRect.topLeft());
+    QRect contentsRect = opt->rect;
+//    painter->translate(contentsRect.topLeft());
     painter->setRenderHint(QPainter::Antialiasing);
 
     painter->save();
@@ -488,6 +514,7 @@ void PreviewWidget::paint(QPainter *painter,
 
     m_logo->paint(painter, QRect(10, 5, s_logoSize, s_logoSize));
 
+    // FIXME: probably too much magic numbers
     painter->save();
     QFont font = KGlobalSettings::smallestReadableFont();
     font.setBold(true);
@@ -495,20 +522,19 @@ void PreviewWidget::paint(QPainter *painter,
     painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
     painter->drawText(QRect(39, 3, contentsRect.width() - 39, 30), Qt::AlignBottom | Qt::AlignLeft,
                       ' ' + i18n("Previewer"));
-    QFontMetrics fm(font);
-    int length = fm.width(i18n("Previewer") + "  ");
+    int length = opt->fontMetrics.width(i18n("Previewer") + "  ");
     font.setBold(false);
     painter->setFont(KGlobalSettings::smallestReadableFont());
-    painter->drawText(QRect(39 + length, 3, contentsRect.width() - 39, 30),
+    QString elidedHint = opt->fontMetrics.elidedText(i18n("Drop files on me to preview them."), Qt::ElideRight, contentsRect.width() - 39 - length - 3);
+    painter->drawText(QRect(39 + length + 3, 3, contentsRect.width() - 39 - length - 3, 30),
                       Qt::AlignBottom | Qt::AlignLeft,
-                      i18n("Drop files on me to preview them."));
+                      elidedHint);
     painter->restore();
 
     drawOpenCloseArrow(painter);
 
     if (!m_items.isEmpty() && m_itemsRect.isValid()) {
         painter->save();
-        painter->setClipping(true);
         painter->setClipRect(m_itemsRect);
 
         PreviewItemModel model(m_previewHistory);
@@ -525,13 +551,20 @@ void PreviewWidget::paint(QPainter *painter,
             }
 
             m_option.state &= ~(QStyle::State_Selected | QStyle::State_MouseOver) ;
+	    m_hoverSvg->setElementPrefix("");
             if (m_selectedIndex == i) {
-                m_option.state |= QStyle::State_Selected;
+                m_hoverSvg->prefix().isEmpty() ? m_hoverSvg->setElementPrefix("selected") : m_hoverSvg->setElementPrefix(m_hoverSvg->prefix() + "+selected");
             }
 
             if (m_hoveredIndex == i) {
-                m_option.state |= QStyle::State_MouseOver;
+                m_hoverSvg->prefix().isEmpty() ? m_hoverSvg->setElementPrefix("hover") : m_hoverSvg->setElementPrefix(m_hoverSvg->prefix() + "+hover");
             }
+	    
+	    if (!m_hoverSvg->prefix().isEmpty()) {
+	        m_hoverSvg->resizeFrame(r.size());
+                m_hoverSvg->paintFrame(painter, r.topLeft());
+                KIcon("list-remove").paint(painter, r.right() - REMOVE_EMBLEM_SIZE, r.y(), REMOVE_EMBLEM_SIZE, REMOVE_EMBLEM_SIZE);
+	    }
 
             m_option.rect = r;
             m_delegate.setShadowColor(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
@@ -543,16 +576,9 @@ void PreviewWidget::paint(QPainter *painter,
     }
 
     // hovering rect
-    if (m_previewReady && !m_hoveredUrl.isEmpty()) {
+    if (!m_hoveredUrl.isEmpty() && m_previews.contains(m_hoveredUrl)) {
         QPixmap pmap;
-        if (m_previews.contains(m_hoveredUrl)) {
-            pmap = m_previews[m_hoveredUrl];
-            QRect itemNameRect = QRect(m_itemsRect.right(), m_itemsRect.bottom(),
-                                       contentsRect.width() - m_itemsRect.right(), bottomBorder);
-            // here we paint the file name
-            painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
-            painter->drawText(itemNameRect, Qt::AlignVCenter | Qt::AlignLeft, m_hoveredUrl.fileName());
-        }
+        pmap = m_previews[m_hoveredUrl];
 
         if (m_previewRect.width() >= pmap.width() && m_previewRect.height() >= pmap.height()) {
             QRect p_rect = QRect(m_previewRect.x() + abs(m_previewRect.width() - pmap.width())/2,
@@ -621,3 +647,36 @@ int PreviewWidget::suggestedWidth()
 
     return (39 + length + fm.width(i18n("Drop files on me to preview them.")) + 70);
 }
+
+QList<QUrl> PreviewWidget::previews()
+{
+    return m_previewHistory;
+}
+
+void PreviewWidget::removeItem(int index)
+{
+    m_previewHistory.removeAt(index);
+    m_selectedIndex = -1;
+    m_hoveredIndex = -1;
+    m_layoutIsValid = false;
+    if (m_previewHistory.isEmpty()) {
+        contract();
+    }
+    lookForPreview();
+    update();
+}
+
+void PreviewWidget::updateMargins()
+{
+    if (!m_panel) {
+        return;
+    }
+
+    qreal left;
+    qreal top;
+    qreal right;
+    qreal bottom;
+
+    m_panel->getMargins(left, top, right, bottom);
+    setContentsMargins(left, top, right, bottom);
+}
